package cn.jlsxwkj.common.reader;

import cn.jlsxwkj.common.utils.SplitDocument;
import lombok.Data;
import org.apache.tika.metadata.Metadata;
import org.apache.tika.parser.AutoDetectParser;
import org.apache.tika.parser.ParseContext;
import org.apache.tika.sax.BodyContentHandler;
import org.springframework.ai.document.Document;
import org.springframework.ai.document.DocumentReader;
import org.springframework.ai.reader.ExtractedTextFormatter;
import org.springframework.core.io.DefaultResourceLoader;
import org.springframework.core.io.Resource;
import org.xml.sax.ContentHandler;

import java.io.InputStream;
import java.util.List;

/**
 * @author zh
 * 文档读取
 */
@Data
public class ParagraphDocReader implements DocumentReader {

    private final AutoDetectParser parser;
    private final ContentHandler handler;
    private final Metadata metadata;
    private final ParseContext context;
    private final Resource resource;
    private int windowSize;
    private ExtractedTextFormatter textFormatter;

    /**
     * Constructor initializing the reader with a given resource URL.
     * @param resourceUrl URL to the resource
     */
    public ParagraphDocReader(String resourceUrl,
                              int windowSize) {
        this(resourceUrl, ExtractedTextFormatter.defaults());
        this.windowSize = windowSize;
    }

    /**
     * Constructor initializing the reader with a given resource URL and a text formatter.
     * @param resourceUrl URL to the resource
     * @param textFormatter Formatter for the extracted text
     */
    public ParagraphDocReader(String resourceUrl,
                              ExtractedTextFormatter textFormatter) {
        this(new DefaultResourceLoader().getResource(resourceUrl), textFormatter);
    }

    /**
     * Constructor initializing the reader with a resource and a text formatter. This
     * constructor will create a BodyContentHandler that allows for reading large PDFs
     * (constrained only by memory)
     * @param resource Resource pointing to the document
     * @param textFormatter Formatter for the extracted text
     */
    public ParagraphDocReader(Resource resource,
                              ExtractedTextFormatter textFormatter) {
        this(resource, new BodyContentHandler(-1), textFormatter);
    }

    /**
     * Constructor initializing the reader with a resource, content handler, and a text
     * formatter.
     * @param resource Resource pointing to the document
     * @param contentHandler Handler to manage content extraction
     * @param textFormatter Formatter for the extracted text
     */
    public ParagraphDocReader(Resource resource,
                              ContentHandler contentHandler,
                              ExtractedTextFormatter textFormatter) {
        this.textFormatter = textFormatter;
        this.parser = new AutoDetectParser();
        this.handler = contentHandler;
        this.metadata = new Metadata();
        this.context = new ParseContext();
        this.resource = resource;
    }

    /**
     * Extracts and returns the list of documents from the resource.
     * @return List of extracted {@link Document}
     */
    @Override
    public List<Document> get() {
        try (InputStream stream = this.resource.getInputStream()) {
            this.parser.parse(stream, this.handler, this.metadata, this.context);
            return SplitDocument.splitStringToListDocument(this.handler.toString(), this.windowSize, this.resource.getFilename());
        }
        catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

}
